@@ -21,7 +21,7 @@ import saver
 import sudoku_generator_gui
 import sudoku_maker
 import timer
-from defaults import (APPNAME, APPNAME_SHORT, AUTHORS, COPYRIGHT, DESCRIPTION,
+from defaults import (APPNAME, APPNAME_SHORT, AUTHORS, COPYRIGHT, DESCRIPTION, DOMAIN, 
         IMAGE_DIR, LICENSE, MIN_NEW_PUZZLES, UI_DIR, VERSION, WEBSITE, WEBSITE_LABEL)
 from gtk_goodies import gconf_wrapper, Undo, dialog_extras
 from simple_debug import simple_debug, options
@@ -140,13 +140,11 @@ class UI (gconf_wrapper.GConfWrapper):
     </ui>'''
 
     initial_prefs = {'group_size':9,
-                     'always_show_hints':0,
-                     'player':os.environ.get('USERNAME', ''),
+                     'always_show_hints':False,
                      'difficulty':0.0,
                      'minimum_number_of_new_puzzles':MIN_NEW_PUZZLES,
-                     'highlight':0,
-                     'bg_black':1,
-                     'bg_custom_color':'',
+                     'highlight':False,
+                     'bg_color':'black',
                      'show_tracker':False,
                      'width': 700,
                      'height': 675,
@@ -334,13 +332,8 @@ class UI (gconf_wrapper.GConfWrapper):
 
     def setup_color (self):
         # setup background colors
-        if self.gconf['bg_custom_color']:
-            bgcol = self.gconf['bg_custom_color']
-        elif self.gconf['bg_black']:
-            bgcol = 'black'
-        else:
-            bgcol = None
-        if bgcol:
+        bgcol = self.gconf['bg_color']
+        if bgcol != '':
             self.gsd.set_bg_color(bgcol)
 
     def setup_autosave (self):
@@ -450,7 +443,6 @@ class UI (gconf_wrapper.GConfWrapper):
                 self.gconf[k]
             except:
                 self.gconf[k] = v
-        self.player = self.gconf['player']
 
     @simple_debug
     @inactivate_new_game_etc
@@ -520,11 +512,6 @@ class UI (gconf_wrapper.GConfWrapper):
         # otherwise we appear sluggish.
         while gtk.events_pending():
             gtk.main_iteration()
-        if self.won:
-            self.gconf['current_game'] = ''
-        if not self.won:
-            if not self.gsd.grid:
-                self.gconf['current_game'] = ''
         self.stop_worker_thread()
         # allow KeyboardInterrupts, which calls quit_cb outside the main loop
         try:
@@ -759,6 +746,7 @@ class TrackerBox (gtk.VBox):
 
         gtk.VBox.__init__(self)
         self.builder = gtk.Builder()
+        self.builder.set_translation_domain(DOMAIN)
         self.builder.add_from_file(os.path.join(UI_DIR, 'tracker.ui'))
         self.main_ui = main_ui
         self.vb = self.builder.get_object('vbox1')
