@@ -34,7 +34,6 @@
 #include <libgames-support/games-conf.h>
 #include <libgames-support/games-help.h>
 #include <libgames-support/games-runtime.h>
-#include <libgames-support/games-sound.h>
 #include <libgames-support/games-stock.h>
 
 #ifdef WITH_SMCLIENT
@@ -42,8 +41,8 @@
 #endif /* WITH_SMCLIENT */
 
 #ifdef GGZ_CLIENT
-#include <games-dlg-chat.h>
-#include <games-dlg-players.h>
+#include <libgames-support/games-dlg-chat.h>
+#include <libgames-support/games-dlg-players.h>
 #include "ggz-network.h"
 #include <ggz-embed.h>
 #endif /* WITH GGZ_CLIENT */
@@ -284,8 +283,8 @@ properties_cb (GtkWidget * widget, gpointer data)
 gint
 expose_event (GtkWidget * widget, GdkEventExpose * event)
 {
-  gdk_draw_drawable (widget->window,
-		     widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
+  gdk_draw_drawable (gtk_widget_get_window (widget),
+		     gtk_widget_get_style (widget)->fg_gc[gtk_widget_get_state (widget)],
 		     buffer_pixmap,
 		     event->area.x, event->area.y,
 		     event->area.x, event->area.y,
@@ -355,7 +354,7 @@ button_press_event (GtkWidget * widget, GdkEventButton * event)
 void
 gui_draw_pixmap (gint which, gint x, gint y)
 {
-  gdk_draw_drawable (drawing_area->window, gridGC[0], tiles_pixmap,
+  gdk_draw_drawable (gtk_widget_get_window (drawing_area), gridGC[0], tiles_pixmap,
 		     (which % 8) * TILEWIDTH, (which / 8) * TILEHEIGHT,
 		     x * (TILEWIDTH + GRIDWIDTH),
 		     y * (TILEHEIGHT + GRIDWIDTH), TILEWIDTH, TILEHEIGHT);
@@ -388,8 +387,8 @@ gui_draw_grid (void)
 		   BOARDWIDTH, i * BOARDHEIGHT / 8 - 1);
   }
 
-  gdk_draw_drawable (drawing_area->window, gridGC[0], buffer_pixmap,
-		     0, 0, 0, 0, BOARDWIDTH, BOARDHEIGHT);
+  gdk_draw_drawable (gtk_widget_get_window (drawing_area), gridGC[0],
+		     buffer_pixmap, 0, 0, 0, 0, BOARDWIDTH, BOARDHEIGHT);
 }
 
 void
@@ -429,7 +428,7 @@ load_pixmaps (void)
 						  &tiles_pixmap,
 						  &tiles_mask, 127);
 
-  gdk_pixbuf_unref (image);
+  g_object_unref (image);
   g_free (fname);
 }
 
@@ -682,17 +681,17 @@ check_computer_players (void)
     switch (black_computer_level) {
     case 1:
       black_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_1,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_1,
 		       (gpointer) BLACK_TURN);
       break;
     case 2:
       black_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_2,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_2,
 		       (gpointer) BLACK_TURN);
       break;
     case 3:
       black_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_3,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_3,
 		       (gpointer) BLACK_TURN);
       break;
     }
@@ -701,17 +700,17 @@ check_computer_players (void)
     switch (white_computer_level) {
     case 1:
       white_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_1,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_1,
 		       (gpointer) WHITE_TURN);
       break;
     case 2:
       white_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_2,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_2,
 		       (gpointer) WHITE_TURN);
       break;
     case 3:
       white_computer_id =
-	g_timeout_add (computer_speed, (GSourceFunc) computer_move_3,
+	add_timeout (computer_speed, (GSourceFunc) computer_move_3,
 		       (gpointer) WHITE_TURN);
       break;
     }
@@ -719,6 +718,17 @@ check_computer_players (void)
   return TRUE;
 }
 
+guint
+add_timeout (guint time, GSourceFunc func, gpointer turn)
+{
+  if (time % 1000) {
+    return g_timeout_add (time, func, turn);
+  } else {
+    time = time / 1000;
+    return g_timeout_add_seconds (time, func, turn);
+  }
+}
+
 void
 set_bg_color (void)
 {
@@ -727,17 +737,17 @@ set_bg_color (void)
 
   tmpimage = gdk_drawable_get_image (tiles_pixmap, 0, 0, 1, 1);
   bgcolor.pixel = gdk_image_get_pixel (tmpimage, 0, 0);
-  gdk_window_set_background (drawing_area->window, &bgcolor);
+  gdk_window_set_background (gtk_widget_get_window (drawing_area), &bgcolor);
 
   if (gridGC[0])
     g_object_unref (gridGC[0]);
-  gridGC[0] = gdk_gc_new (drawing_area->window);
+  gridGC[0] = gdk_gc_new (gtk_widget_get_window (drawing_area));
   if (gridGC[1])
     g_object_unref (gridGC[1]);
-  gridGC[1] = gdk_gc_new (drawing_area->window);
+  gridGC[1] = gdk_gc_new (gtk_widget_get_window (drawing_area));
 
-  gdk_gc_copy (gridGC[0], drawing_area->style->bg_gc[0]);
-  gdk_gc_copy (gridGC[1], drawing_area->style->bg_gc[0]);
+  gdk_gc_copy (gridGC[0], gtk_widget_get_style (drawing_area)->bg_gc[0]);
+  gdk_gc_copy (gridGC[1], gtk_widget_get_style (drawing_area)->bg_gc[0]);
 
   gdk_gc_set_background (gridGC[0], &bgcolor);
   gdk_gc_set_foreground (gridGC[0], &bgcolor);
@@ -764,7 +774,7 @@ save_state_cb (EggSMClient *client,
   gint argc;
   gint xpos, ypos;
 
-  gdk_window_get_origin (window->window, &xpos, &ypos);
+  gdk_window_get_origin (gtk_widget_get_window (window), &xpos, &ypos);
 
   argc = 0;
   argv[argc++] = g_get_prgname ();
@@ -991,7 +1001,6 @@ main (int argc, char **argv)
   g_option_context_add_group (context, egg_sm_client_get_option_group ());
 #endif /* WITH_SMCLIENT */
   g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-  games_sound_add_option_group (context);
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_option_context_free (context);
@@ -1020,7 +1029,7 @@ main (int argc, char **argv)
   create_window ();
 
   gtk_widget_show (window);
-  buffer_pixmap = gdk_pixmap_new (drawing_area->window,
+  buffer_pixmap = gdk_pixmap_new (gtk_widget_get_window (drawing_area),
 				  BOARDWIDTH, BOARDHEIGHT, -1);
 
   load_properties ();
@@ -1032,7 +1041,7 @@ main (int argc, char **argv)
 #endif /* GGZ_CLIENT */
 
   if (session_xpos >= 0 && session_ypos >= 0) {
-    gdk_window_move (window->window, session_xpos, session_ypos);
+    gdk_window_move (gtk_widget_get_window (window), session_xpos, session_ypos);
   }
 
   set_bg_color ();
